/***************************************************************************
 *            utrac_cmd.c
 *
 *  Tue Oct  5 11:30:06 2004
 *  Copyright  2004  Antoine Calando
 *  antoine@alliancemca.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/*!
 * \file utrac_cmd.c
 * \author Antoine Calando (antoine@alliancemca.net)
 * \brief "utrac" shell command.
 * \todo doc dixygen...
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "utrac.h"

#undef UT_DEBUG
#define UT_DEBUG 3
#include "debug.h"

#define ESC_RED "\e[31m"
#define ESC_NONE "\e[0m"

int custom_pb_fct (float rate) {
	
	if (!rate) printf("Rate = 0\n");
		
	const int barsize = 100;
	
	putchar(0xD);
	float i; for (i=0; i < barsize*rate; i++) {
		putchar('*');
		//printf ("<%f>", i);
	}
	for (; i<barsize; i++) putchar ('-');
	putchar(0xD);
	
	//printf ("%2.0f%%\n", (double) 100*rate);
	if (rate==1.0) printf ("\n fin!\n");
	
	return true;
}



UtCode parse_charset_eol (char * charset_eol, int *charset_i, UtEolType * eol_type) {
	
	int i;
	for (i=0; i<ut_session->charset_n; i++)
		if ( ut_str_fuzzy_cmp (charset_eol, ut_session->charset[i].name, '/')) break;
	
	if (i==ut_session->charset_n) *charset_i = UT_UNSET;
	else {
		*charset_i=i;
		while (*charset_eol!='/' && *charset_eol) charset_eol++;
		if (!*charset_eol) {
			*eol_type = UT_EOL_UNSET;	
			return UT_OK;
		}
		charset_eol++;
	}
    
	UtEolType j;
	for (j=UT_EOL_CRLF; j<UT_EOL_NON_STD; j++) {
		if ( !strcasecmp (charset_eol, UT_EOL_NAME[j]) ) {
			*eol_type = j;
			return UT_OK;
		}
	}

	return UT_NOT_FOUND_ERROR;
}

void test_rcode (UtCode rcode, char * argv[]) {
	if (rcode!=UT_OK) {
		fprintf (stderr, "%s : error %d\n", argv[0], rcode);
		if (ut_session->error_string[0]) fprintf (stderr, "%s: %s\n", argv[0], ut_session->error_string);
		exit(rcode);	
	}
}



int main (int argc, char * argv[]) {
	
	int i,j;
	
	for (i=1; i<argc; i++) {
		
		//if (argv[i][0] =='-') {
		if (!strcmp (argv[i], "-h") ||	!strcmp (argv[i], "--help")) {
			fprintf (stderr, 
			"usage: %s [OPTION] [FILE]\n"
			"With no FILE, read standard input; with no OPTION, write converted text to standard output.\n"
			"LC_ALL, LC_TYPE, LANG are read to determine prefered language and output charset.\n"
			"\n"
			"	-f	--from			Force input charset (disable recognition)\n"
			"	-t	--to			Select output charset\n"
			"	-x	--ext-chars		Print lines with extended characters\n"
			"	-c	--colors		(with -x) Use colors\n"
			"	-p	--print-charset		Print recognized charset\n"
			"	-P	--print-all-charset	Print ranked list of charsets\n"
			"\n",
			argv[0]
			);
			exit(0);
		} else if (!strcmp (argv[i], "-v") ||	!strcmp (argv[i], "--version")) {
			fprintf (stderr, "Utrac Universal Text Recognizer And Converter" UT_VERSION "\n"
					"Written by Antoine Calando - Alliance MCA\n");
			exit(0);
		} else if (!strcmp (argv[i], "-l") ||	!strcmp (argv[i], "--list")) {
			UtCode rcode = ut_init ();
			test_rcode (rcode,argv);
			for (j=0; j<ut_session->charset_n; j++) fprintf (stdout, "%s\n", ut_session->charset[j].name);
			ut_finish ();
			exit (0);
		}
	}

	UtCode rcode = ut_init ();
	test_rcode (rcode,argv);
	
	UtText * text = ut_init_text ();
	
	int src_charset = UT_UNSET, dst_charset = UT_UNSET;
	UtEolType src_eol = UT_EOL_UNSET, dst_eol = UT_EOL_UNSET;
	bool print_ext_char = false;
	bool use_color = false;
	bool print_charset_name = false;
	bool print_all_charsets = false;
	
	
	for (i=1; i<argc; i++) {
		if (!strcmp (argv[i], "-f") ||	!strcmp (argv[i], "--from")) {
			
			if (++i==argc || parse_charset_eol (argv[i], &src_charset, &src_eol)!=UT_OK) {
				fprintf (stderr, "%s : error invalid charset %s\nTry `%s --help' for more information\n", argv[0], argv[i], argv[0]);
				ut_finish (); exit(-1);
			}
			if (src_eol!=UT_EOL_UNSET) {
				fprintf (stderr, "%s : warning input EOL type is ignored\n", argv[0]);
			}
		} else if (!strcmp (argv[i], "-t") ||	!strcmp (argv[i], "--to")) {
			if (++i==argc || parse_charset_eol (argv[i], &dst_charset, &dst_eol)!=UT_OK) {
				fprintf (stderr, "%s : error invalid charset %s\nTry `%s --help' for more information\n", argv[0], argv[i], argv[0]);
				ut_finish (); exit(-1);
			}
		} else if (!strcmp (argv[i], "-x") ||	!strcmp (argv[i], "--ext-chars")) {
			print_ext_char = true;
		} else if (!strcmp (argv[i], "-c") ||	!strcmp (argv[i], "--colors")) {
			use_color = true;
		} else if (!strcmp (argv[i], "-p") || !strcmp (argv[i], "--print-charset")) {
			print_charset_name = true;
		} else if (!strcmp (argv[i], "-P") || !strcmp (argv[i], "--print-all-charsets")) {
			print_all_charsets = true;
		} else if (!text->filename) {
			text->filename = argv[i];
		} else {
			fprintf (stderr, "%s : error invalid option %s\nTry `%s --help' for more information\n", argv[0], argv[i], argv[0]);
			ut_finish (); exit(-1);
		}
	}
	
	if (use_color) print_ext_char = true;
	#if UT_DEBUG==3
	if (src_charset!=UT_UNSET) fprintf (stderr,"DEBUG Input charset selected : %s\n", ut_session->charset[src_charset].name);
	else fprintf (stderr,"DEBUG No input charset selected\n");
	if (src_eol!=UT_EOL_UNSET) fprintf (stderr,"DEBUG Input eol type selected : %s\n", UT_EOL_NAME [src_eol]);
	else fprintf (stderr,"DEBUG No input eol type selected\n");
	if (dst_charset!=UT_UNSET) fprintf (stderr,"DEBUG Output charset selected : %s\n", ut_session->charset[dst_charset].name);
	else fprintf (stderr,"DEBUG No output charset selected\n");
	if (dst_eol!=UT_EOL_UNSET) fprintf (stderr,"DEBUG Output eol type selected : %s\n", UT_EOL_NAME [dst_eol]);
	else fprintf (stderr,"DEBUG No Output eol type selected\n");
	#endif
	
	if (dst_charset!=UT_UNSET) text->dst_charset = dst_charset;
	if (dst_eol!=UT_EOL_UNSET) text->dst_eol = dst_eol;

	if (src_charset!=UT_UNSET) text->src_charset = src_charset;
	if (src_eol!=UT_EOL_UNSET) text->src_eol = src_eol;

	if (print_ext_char || print_charset_name || print_all_charsets) {
		rcode = ut_process_text (text, false);
		test_rcode (rcode,argv);

		if (print_ext_char) {
			UtExtCharLine *scan_exl = text->ext_char;
			char * scan_char;
			while (scan_exl) {
				printf ("%3lu [%lu]: ", scan_exl->line_i, scan_exl->ext_char_n);
				scan_char = scan_exl->line_p;
				while (*scan_char) {
					if ((uchar)*scan_char<0x80) putchar(*scan_char++);
					else {
						char dst_char[5]; //max utf-8 length is 4
						char *dst_char_p = dst_char;
						ut_conv_char (text, &scan_char, &dst_char_p);
						*dst_char_p = 0;
						if (use_color) printf (ESC_RED);
						printf (dst_char);
						if (use_color) printf (ESC_NONE);
						
					//if (*scan_char>0x9F) printf ("\e[31m%c\e[0m", (int) *scan_char);
					//else printf ("\e[31m%c\e[0m", (int) *scan_char - 0x80 +'A');
					}
					//scan_char++;
				}
				putchar ('\n');
				scan_exl = scan_exl->next;	
			}
			
		} 
		
		if (print_charset_name || (print_all_charsets && !text->evaluation)) {
			printf ("charset %d: %s\n", text->reco_charset, ut_session->charset [text->reco_charset].name);
			
		}
		
		if (print_all_charsets && text->evaluation) {
			
			ulong prev_checksum = 0;
			long prev_rating = LONG_MIN; 
			
			bool *done = malloc (sizeof(bool)*ut_session->charset_n);
			int i; for (i=0; i<ut_session->charset_n; i++) done[i] = false;
				
			//printf ("%8ld\n", LONG_MIN);
				
			for (;;) {
				long rating_max = LONG_MIN; 
				short index = -1;
				
				for (i=0; i<ut_session->charset_n; i++) {
					if (done[i]) continue;
					//printf ("<%d>", i);	
					if (text->evaluation[i].rating == prev_rating ) {
						rating_max = text->evaluation[i].rating;
						index = i;
						//printf ("*");
						if (text->evaluation[i].checksum == prev_checksum) break;
					} else if (text->evaluation[i].rating > rating_max) {
						rating_max = text->evaluation[i].rating;
						index = i;
						//printf ("+");
					}
				}
				if (index == -1) break;
				if ( i == ut_session->charset_n) {
					printf ("\n %8ld (%8lx) : %s", text->evaluation [index].rating, text->evaluation [index].checksum,
							ut_session->charset [index].name );
				} else {
					printf (", %s", ut_session->charset [index].name);
				}
				prev_checksum = text->evaluation [index].checksum;
				prev_rating = rating_max;
				done[index] = true;
				
			} //for (;;)
			printf ("\n");
		} 
	} else {
		rcode = ut_process_text (text, true);
		test_rcode (rcode,argv);

		printf (text->data);
	}

	ut_free_text (text);
	
	ut_finish ();
	exit (0);
	
	
	/*
	
	UtCode rcode = ut_init ();
	
	if (rcode!=UT_OK) {
		printf ("Error! ");
		if (ut_session->error_string[0]) printf (ut_session->error_string);
		else printf (" num : %d", rcode);
		putchar('\n');
		return rcode;	
	}
	
	
	if (argc==2) {
	
		UtText * text = ut_init_text ();
		
		if (strcmp(argv[1],"-")) {
			DBG3 ("Loading : <%s>", argv[1])
			text->filename = argv[1];
		} else {
			DBG3 ("stdin!")
			text->filename = NULL;
		}
		text->progress_function = custom_pb_fct;
			
		rcode = ut_process_text (text, true);
		if (rcode!=UT_OK) {
			printf ("Error! ");
			if (ut_session->error_string[0]) printf (ut_session->error_string);
			else printf (" num : %d", rcode);
			putchar('\n');
			return rcode;
		}
		
		ut_debug_text (text);
		//ut_debug_text_rating (text);

		//text->dst_charset = 4;
		//text->dst_eol = UT_EOL_CRLF;
		//text->nomapping_char = UT_UNICODE_NONCHAR; //0x7F; //
		//ut_conversion_pass (text);
		
		printf ("=============== conversion de %s vers %s ================\n",
			ut_session->charset [text->src_charset].name, ut_session->charset [text->dst_charset].name);
		//printf (text->data);
		
		ut_free_text(text);
	} //if argc==1
	
	ut_finish ();
	
	return UT_OK;
	*/
	
} //end main
